import {
    world,
    system,
    ItemStack
} from "@minecraft/server"
import {
    ModalFormData,
    ActionFormData
} from "@minecraft/server-ui"
import {
    getHealth,
    amulets,
    showMana,
    teleportSound
} from "./functions.js"
import {
    spells,
    staves,
    attackStaves,
    scrolls
} from "./spells.js"

export let Mana = 30; //Determines how much mana the player has
const Version = "1.3.0" //Current MagiCraft version. Decides if a changelog will spawn for the player.


console.warn("Succesfully Loaded Script");

world.afterEvents.entityHurt.subscribe((event) => {
    const source = event.damageSource;
    const target = event.hurtEntity;
    const damage = event.damage / 5;

    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")
        const head = player?.getComponent("equippable").getEquipment("Head")
        const chest = player?.getComponent("equippable").getEquipment("Chest")
        const legs = player?.getComponent("equippable").getEquipment("Legs")
        const feet = player?.getComponent("equippable").getEquipment("Feet")

        //Display health of entities if under 50 total health
        if (target?.hasComponent("health")) {
            if (target.getComponent("minecraft:health").effectiveMax <= 50 && source.damagingEntity == player) {
                const health = "|".repeat(target.getComponent("minecraft:health").currentValue.toFixed(0))
                const whitespace = "|".repeat(target.getComponent("minecraft:health").effectiveMax.toFixed(0) - target.getComponent("minecraft:health").currentValue.toFixed(0))
                const mana = "|".repeat(Mana)
                const whitespace2 = "|".repeat((30 - Mana))

                player.onScreenDisplay.setActionBar("[§c" + health + "§r" + whitespace + "] / [§9" + mana + "§r" + whitespace2 + "]")
            }
        }

        //Restore mana when using mythril
        if (item?.getComponent("enchantable")) {
            if (item?.getComponent("enchantable").getEnchantment("mending") && item?.hasTag("magicraft:is_mythril") && (source.cause == "entityAttack" || source.cause == "contact") && source.damagingEntity == player) {
                Mana += 1;
                if (Mana > 30) {
                    Mana = 30;
                }
            }
        }

        //Weapon cooldown & sweeping attacks
        if (item?.hasTag("magicraft:is_saber") === true && source.cause === "entityAttack" && source.damagingEntity == player) {
            player.addEffect("weakness", 12, { showParticles: false, amplifier: 2 })
            if (target.location.y <= player.location.y + 2 && player.isOnGround == true && player.isSprinting == false) {
                target.addTag("target")
                target.runCommand("damage @e[tag=!target,type=!player,r=1.75] " + damage.toFixed(0) + " contact entity @p")
                target.runCommand("particle magicraft:sweep1 ~ ~1 ~")
                target.removeTag("target")
            }
        }
        else if (item?.hasTag("minecraft:is_sword") == true && source.cause == "entityAttack" && source.damagingEntity == player) {
            player.addEffect("weakness", 15, { showParticles: false, amplifier: 2 })
        }
        else if (item?.hasTag("minecraft:is_axe") == true && source.cause == "entityAttack" && source.damagingEntity == player) {
            player.addEffect("weakness", 24, { showParticles: false, amplifier: 2 })
        }
        else if (item?.hasTag("magicraft:is_battleaxe") == true && source.cause == "entityAttack" && source.damagingEntity == player) {
            player.addEffect("weakness", 29, { showParticles: false, amplifier: 4 })
            player.addEffect("slowness", 29, { showParticles: false, amplifier: 0 })
        }
        else if (item?.hasTag("magicraft:is_greatsword") == true && source.cause == "entityAttack" && source.damagingEntity == player) {
            player.addEffect("weakness", 24, { showParticles: false, amplifier: 3 })
            player.addEffect("slowness", 24, { showParticles: false, amplifier: 0 })
        }
    }

    //Fall damage causes slowness
    if (source.cause == "fall" && event.damage >= 5 && event.damage < 10) {
        target.addEffect("slowness", 100, { showParticles: false, amplifier: 1 })
    }
    else if (source.cause == "fall" && event.damage >= 10) {
        target.addEffect("slowness", 100, { showParticles: false, amplifier: 1 })
        target.addEffect("mining_fatigue", 100, { showParticles: false, amplifier: 1 })
    }
})

world.afterEvents.projectileHitEntity.subscribe((event) => {
    const source = event.source
    const target = event.getEntityHit().entity
    const location = event.location.y

    for (const player of world.getPlayers()) {
        const offhand = player?.getComponent("equippable").getEquipment("Offhand")
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        if (target == player) {
            const targethit = target.getHeadLocation().y
            const helmet = player?.getComponent("equippable").getEquipment("Head")

            //Give blindness to player if struck in the head and they're not wearing a helmet or blocking with a shield
            if (helmet?.typeId == null) {
                if (location >= targethit && (player.isSneaking == true && (offhand?.typeId == "minecraft:shield" || item?.typeId == "minecraft:shield"))) {
                    return;
                }
                else if (location >= targethit) {
                    player.addEffect("blindness", 100, { showParticles: false, amplifier: 0 })
                }
            }
            else if (helmet?.typeId != null) {
                return;
            }
        }
        /*else if (target != player && target != "minecraft:warden") {
            const targethit = target.getHeadLocation().y
            world.sendMessage(location + " + " + targethit)

            if (location >= targethit) {
                target.addEffect("blindness", 100, { showParticles: true, amplifier: 0 })
            }
        }*/ //Currently a WIP- trtying to figure out how to make mobs non-hostile
    }
})

world.afterEvents.itemStopUse.subscribe((event) => {
    const chargeTime = event.useDuration;

    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        //Cancel casting of spells if not charged enough
        if ((item?.hasTag("magicraft:is_spellbook") || item?.hasTag("magicraft:is_scroll")) && chargeTime > 19960) {
            player.playSound("item.book.put")
        }
        if ((item?.hasTag("magicraft:is_spell") || item?.hasTag("magicraft:is_staff")) && chargeTime > 19960) {
            player.playSound("beacon.deactivate")
        }
        if (item?.hasTag("magicraft:is_attack_staff") && chargeTime > 19980) {
            player.playSound("beacon.deactivate")
        }
    }
})

world.afterEvents.itemReleaseUse.subscribe((event) => {
    const chargeTime = event.useDuration;

    for (const player of world.getPlayers()) {

        //Set minimum xp given to 1
        const spellXp = Math.max(1, (player.totalXpNeededForNextLevel * 0.0588));

        //Get mainhand item
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        //Get offhand item
        const offhand = player?.getComponent("equippable").getEquipment("Offhand")

        //Spell casting sounds
        if ((item?.hasTag("magicraft:is_spell") || item?.hasTag("magicraft:is_staff")) && chargeTime <= 19960) {
            player.playSound("mob.evocation_illager.cast_spell")
        }
        if (item?.hasTag("magicraft:is_attack_staff") && chargeTime <= 19980) {
            player.playSound("mob.evocation_illager.cast_spell")
        }

        //Spellbook sounds
        if ((item?.hasTag("magicraft:is_spellbook") || item?.hasTag("magicraft:is_scroll")) && chargeTime <= 19960) {
            player.playSound("block.enchanting_table.use")
        }

        //Reduce Catalyst durability if spell is cast
        if ((item?.hasTag("magicraft:is_spell") && offhand?.typeId == "magicraft:darkbloom_catalyst") && chargeTime <= 19960) {
            const damage = offhand.getComponent("durability").damage;
            const durability = offhand.getComponent("durability").maxDurability;
            if (damage == durability) {
                player.getComponent("equippable").getEquipmentSlot("Offhand").setItem(null)
                player.playSound("random.break")
            }
            else {
                offhand.getComponent("durability").damage += 1;
                player.getComponent("equippable").getEquipmentSlot("Offhand").setItem(offhand)
            }
        }


        //Use totem to return home
        if (item?.typeId == "magicraft:totem_home" && chargeTime <= 19960) {
            if (item?.getDynamicProperty("home") != null) {
                if (player.dimension.id == item?.getDynamicProperty("dimension")) {
                    player.teleport(item?.getDynamicProperty("home"))
                    system.runTimeout(teleportSound, 1)
                }
                else {
                    player.onScreenDisplay.setTitle(" ", { fadeInDuration: 5, fadeOutDuration: 5, stayDuration: 80 })
                    player.onScreenDisplay.updateSubtitle("§9Waypoints are not inter-dimensional!")
                }
            }
            else {
                //world.sendMessage("No waypoint found. Set a waypoint by crouching and using this Waypoint Totem!")
                player.onScreenDisplay.setTitle(" ", { fadeInDuration: 5, fadeOutDuration: 5, stayDuration: 80 })
                player.onScreenDisplay.updateSubtitle("§9No waypoint found. \n §9Set a waypoint by crouching \n §9and using this Waypoint Totem!")
            }
        }


        //Use spells

        //With darkbloom catalyst
        if (offhand?.typeId == "magicraft:darkbloom_catalyst" && Mana >= 2) {
            if (item?.hasTag("magicraft:is_spell") == true && chargeTime <= 19960) {
                system.run(spells)
                Mana -= 2;
                player.addExperience(spellXp)
                if (Mana < 0) {
                    Mana = 0;
                }
            }
        }

        //Without darkbloom catalyst
        else if (offhand?.typeId != "magicraft:darkbloom_catalyst" && Mana >= 5) {
            if (item?.hasTag("magicraft:is_spell") == true && chargeTime <= 19960) {
                system.run(spells)
                Mana -= 5;
                player.addExperience(spellXp)
                if (Mana < 0) {
                    Mana = 0;
                }
            }
        }

        //NOE notifications
        else if (offhand?.typeId == "magicraft:darkbloom_catalyst" && item?.hasTag("magicraft:is_spell") && Mana < 2) {
            player.onScreenDisplay.setTitle(" ", { fadeInDuration: 5, fadeOutDuration: 5, stayDuration: 30 })
            player.onScreenDisplay.updateSubtitle("You don't have enough energy!")
        }
        else if (offhand?.typeId != "magicraft:darkbloom_catalyst" && item?.hasTag("magicraft:is_spell") && Mana < 5) {
            player.onScreenDisplay.setTitle(" ", { fadeInDuration: 5, fadeOutDuration: 5, stayDuration: 30 })
            player.onScreenDisplay.updateSubtitle("You don't have enough energy!")
        }


        //Use Spellbooks
        {
            if (item?.typeId == "magicraft:tome_healing" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:heal_spell")
            }
            if (item?.typeId == "magicraft:tome_resistance" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:resistance_spell")
            }
            if (item?.typeId == "magicraft:tome_fireprot" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:fire_protection")
            }
            if (item?.typeId == "magicraft:tome_fang" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:fang_summoning")
            }
            if (item?.typeId == "magicraft:tome_familiar" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:summon_familiar")
            }
            if (item?.typeId == "magicraft:tome_nv" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:night_vision_spell")
            }
            if (item?.typeId == "magicraft:tome_reanimate" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:reanimation_spell")
            }
            if (item?.typeId == "magicraft:tome_wither" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:induce_wither")
            }
            if (item?.typeId == "magicraft:tome_thunder" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:thunder_spell")
            }
            if (item?.typeId == "magicraft:tome_jump" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_jump")
            }
            if (item?.typeId == "magicraft:tome_weakness" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:induce_weakness")
            }
            if (item?.typeId == "magicraft:tome_waffinity" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_waffinity")
            }
            if (item?.typeId == "magicraft:tome_clairvoyance" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_clairvoyance")
            }
            if (item?.typeId == "magicraft:tome_protection" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_protection")
            }
            if (item?.typeId == "magicraft:tome_absorption" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_absorption")
            }
            if (item?.typeId == "magicraft:tome_wmending" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_wmending")
            }
            if (item?.typeId == "magicraft:tome_decay" && chargeTime <= 19960) {
                player.runCommand("replaceitem entity @s slot.weapon.mainhand 0 magicraft:spell_decay")
            }
        }


        //Use scrolls
        if (item?.hasTag("magicraft:is_scroll") && chargeTime <= 19960) {

            system.run(scrolls)

        }


        //Use staves
        if ((item?.hasTag("magicraft:is_staff") || item?.hasTag("magicraft:is_attack_staff")) && Mana >= 2) {
            if (chargeTime <= 19980 && item?.hasTag("magicraft:is_attack_staff")) {
                system.run(attackStaves)
                Mana -= 2;
                if (Mana < 0) {
                    Mana = 0;
                }
            }
            else if (chargeTime <= 19960 && item?.hasTag("magicraft:is_staff")) {
                system.run(staves)
                Mana -= 2;
                if (Mana < 0) {
                    Mana = 0;
                }
            }
        }

        //Not enough energy notification for staves
        else if ((item?.hasTag("magicraft:is_staff") || item?.hasTag("magicraft:is_attack_staff")) && Mana < 2) {
            player.onScreenDisplay.setTitle(" ", { fadeInDuration: 5, fadeOutDuration: 5, stayDuration: 30 })
            player.onScreenDisplay.updateSubtitle("You don't have enough energy!")
        }
    }
})

world.afterEvents.itemStartUse.subscribe((event) => {
    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")
        const mana = "|".repeat(Mana)
        const whitespace = "|".repeat((30 - Mana))

        //Start casting spell
        if (item?.hasTag("magicraft:is_spell") || item?.hasTag("magicraft:is_staff") || item?.hasTag("magicraft:is_attack_staff") || item?.hasTag("magicraft:is_totem")) {
            player.playSound("mob.evocation_illager.prepare_summon")
        }

        //Spellbook start sound
        if (item?.hasTag("magicraft:is_spellbook") || item?.hasTag("magicraft:is_scroll")) {
            player.playSound("item.book.page_turn")
        }

        //Mana notification
        if (item?.hasTag("magicraft:is_spell")) {
            player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
        }
    }
})

world.afterEvents.playerBreakBlock.subscribe((event) => {
    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        //Durability when breaking a block
        if (item?.hasTag("magicraft:custom_tool")) {
            const damage = item.getComponent("durability").damage;
            const durability = item.getComponent("durability").maxDurability;
            const inv = player.getComponent("inventory").container
            if (damage == durability) {
                inv.setItem(player.selectedSlotIndex, null)
                player.playSound("random.break")
            }
            else {
                item.getComponent("durability").damage += 1;
                inv.setItem(player.selectedSlotIndex, item);
            }
        }
    }
})

world.afterEvents.itemUseOn.subscribe((event) => {
    const block = event.block;

    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        //Durability when using a tool
        if (item?.hasTag("minecraft:is_axe") && block.hasTag("wood")) {
            const damage = item.getComponent("durability").damage;
            const durability = item.getComponent("durability").maxDurability;
            const inv = player.getComponent("inventory").container
            if (damage == durability) {
                inv.setItem(player.selectedSlotIndex, null)
                player.playSound("random.break")
            }
            else {
                item.getComponent("durability").damage += 1;
                inv.setItem(player.selectedSlotIndex, item);
            }
        }
        else if (item?.hasTag("minecraft:is_shovel") && block.hasTag("grass")) {
            const damage = item.getComponent("durability").damage;
            const durability = item.getComponent("durability").maxDurability;
            const inv = player.getComponent("inventory").container
            if (damage == durability) {
                inv.setItem(player.selectedSlotIndex, null)
                player.playSound("random.break")
            }
            else {
                item.getComponent("durability").damage += 1;
                inv.setItem(player.selectedSlotIndex, item);
            }
        }
        else if (item?.hasTag("minecraft:is_hoe") && block.hasTag("dirt")) {
            const damage = item.getComponent("durability").damage;
            const durability = item.getComponent("durability").maxDurability;
            const inv = player.getComponent("inventory").container
            if (damage == durability) {
                inv.setItem(player.selectedSlotIndex, null)
                player.playSound("random.break")
            }
            else {
                item.getComponent("durability").damage += 1;
                inv.setItem(player.selectedSlotIndex, item);
            }
        }
    }
})

world.afterEvents.itemCompleteUse.subscribe((event) => {
    const mana = "|".repeat(Mana)
    const whitespace = "|".repeat((30 - Mana))

    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        //Use one durability on custom potions
        if (item?.hasTag("magicraft:custom_potion")) {
            const damage = item?.getComponent("durability").damage;
            const durability = item?.getComponent("durability").maxDurability;
            const inv = player.getComponent("inventory").container

            //Darkbloom potions - used for regenerating mana
            if (item?.typeId == "magicraft:darkbloom_potion" && Mana < 25) {
                Mana += 5
                if (Mana > 30) {
                    Mana = 30;
                }

                if (damage == (durability - 1)) {
                    inv.setItem(player.selectedSlotIndex, new ItemStack("glass_bottle"))
                    Mana += 5
                    if (Mana > 30) {
                        Mana = 30;
                    }
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                }
                else {
                    item.getComponent("durability").damage += 1;
                    inv.setItem(player.selectedSlotIndex, item);
                }
            }
            else if (item?.typeId == "magicraft:darkbloom_potion" && Mana >= 25) {
                Mana += (30 - Mana);


                if (damage == (durability - 1)) {
                    inv.setItem(player.selectedSlotIndex, new ItemStack("glass_bottle"))
                    Mana += (30 - Mana);
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                }
                else {
                    item.getComponent("durability").damage += 1;
                    inv.setItem(player.selectedSlotIndex, item);
                }
            }
        }
    }
})

world.afterEvents.playerSpawn.subscribe((event) => {

    //Run functions on spawn & set player's mana
    system.run(getHealth)
    system.run(amulets)
    system.run(regenMana)
    system.run(showMana)
    Mana = 30;

    for (const player of world.getPlayers()) {
        const version = player.getDynamicProperty("version")

        if (version != Version) {
            player.setDynamicProperty("version", Version)
            player.dimension.spawnItem(new ItemStack("magicraft:changelog"), player.location)
        }
        else {
            return;
        }
    }
})

world.afterEvents.itemUse.subscribe((event) => {
    for (const player of world.getPlayers()) {
        const item = player?.getComponent("equippable").getEquipment("Mainhand")

        if (item?.typeId == "magicraft:totem_home" && player.isSneaking) {
            const Home = player.location;
            let Name = "Waypoint";
            const Dimension = player.dimension.id;
            const homeLocation = Home.x.toFixed(0) + ", " + Home.y.toFixed(0) + ", " + Home.z.toFixed(0);
            const inv = player.getComponent("inventory").container

            const form = new ModalFormData()
                .title("Name Waypoint")
                .textField("Waypoint Name", "Waypoint Name", "Waypoint");
            form.show(player).then(formData => {
                Name = formData.formValues;

                if (Name == undefined) {
                    world.sendMessage("Waypoint creation canceled.")
                }
                else {
                    world.sendMessage("Waypoint coordinates set to " + Home.x.toFixed(0) + ", " + Home.y.toFixed(0) + ", " + Home.z.toFixed(0))
                    item?.setDynamicProperty("home", Home)
                    item?.setDynamicProperty("dimension", Dimension)
                    item?.setLore(["§7" + Name, "§7" + homeLocation, "§7" + Dimension])
                    inv.setItem(player.selectedSlotIndex, item)
                }
            })
        }

        if (item?.typeId == "magicraft:changelog") {
            const form = new ActionFormData()
                .title("MagiCraft V" + Version + " Changes")
                .body("§l§oChanges & Additions \n§r - Being shot in the head now gives blindess. This only affects you if you're not blocking or don't have a helmet on. \n\n - Greatly increased number of blocks custom pickaxes can break. \n\n - Added basic totems & waypoint totems. To use a waypoint totem, sneak and use the item. You'll be prompted to name the totem. Then, charge the totem for two seconds and release to teleport to your destination. \n\n - Greatly increased the number of mobs that Life Essence drops from. \n\n - Spells now have no downsides and static potency. \n\n - A bunch of code cleanup has been done in the background. \n\n---------\n\nPlease leave suggestions for the next update on the CurseForge page! \n\nThank you for trying my addon. \n\n - Rye, the creator of MagiCraft\n\n")
                .button("Delete Changelog");
            form.show(player).then(formData => {
                if (formData.selection == 0) {
                    const inv = player.getComponent("inventory").container
                    const mainhand = player.getComponent("equippable").getEquipment("Mainhand")
                    if (mainhand?.typeId != "magicraft:changelog") {
                        world.sendMessage("Please equip the changelog to remove it from your inventory.")
                    }
                    else if (mainhand?.typeId == "magicraft:changelog") {
                        inv.setItem(player.selectedSlotIndex, null)
                    }
                }
                /*else if (formData.selection == 0 && item?.typeId != "magicraft:changelog") {
                    world.sendMessage("Please equip the changelog to remove it from your inventory.")
                }*/
                else if (formData.selection != 0) {
                    return;
                }
            })
        }
    }
})

function regenMana() {
    for (const player of world.getPlayers()) {
        const mana = "|".repeat(Mana)
        const whitespace = "|".repeat((30 - Mana))

        if (player.getEntitiesFromViewDirection({ maxDistance: 7 }).length == 0) {
            if (player.level < 5) {
                if (system.currentTick % 200 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 5 && player.level < 10) {
                if (system.currentTick % 160 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 10 && player.level < 15) {
                if (system.currentTick % 140 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 15 && player.level < 20) {
                if (system.currentTick % 120 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 20 && player.level < 25) {
                if (system.currentTick % 100 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 25 && player.level < 35) {
                if (system.currentTick % 80 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 35) {
                if (system.currentTick % 40 === 0 && Mana < 30) {
                    Mana += 1;
                    player.onScreenDisplay.setActionBar("[§9" + mana + "§r" + whitespace + "]")
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
        }
        else if (player.getEntitiesFromViewDirection({ maxDistance: 7 }).length != 0) {
            if (player.level < 5) {
                if (system.currentTick % 200 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 5 && player.level < 10) {
                if (system.currentTick % 160 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 10 && player.level < 15) {
                if (system.currentTick % 140 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 15 && player.level < 20) {
                if (system.currentTick % 120 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 20 && player.level < 25) {
                if (system.currentTick % 100 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 25 && player.level < 35) {
                if (system.currentTick % 80 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
            else if (player.level >= 35) {
                if (system.currentTick % 40 === 0 && Mana < 30) {
                    Mana += 1;
                    system.run(regenMana)
                }
                else {
                    system.run(regenMana)
                }
            }
        }
    }
}
system.run(getHealth)
system.run(amulets)
system.run(regenMana)
system.run(showMana)